////////////////////////////////////////////////////////////////////////////
//
// SerialPort to I2C-FM Transmitter(NS73M) Test Program.
//
//   Copyright by rubidesu 2013
//
//   http://www.gatelink.co.jp/hw/
//
//   Version 1.00 2016.02.20
//
////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <windows.h>
#include <conio.h>

#include "i2c.h"

#define DEFAULT_FREQ	800

#define SLEEP_TIME		10		//msecP.

const BYTE bDeviceAddr = 0xce;

////////////////////////////////////////////////////////////////////////////
BOOL LibInitialize()
{
	HINSTANCE hDll;

	if((hDll = LoadLibrary("i2c.dll")) == NULL){
		return FALSE;
	}
	else{
		if((i2c_start = (I2C_START)GetProcAddress(hDll,"i2c_start")) == NULL){
			FreeLibrary(hDll);
			return FALSE;
		}
		if((i2c_stop = (I2C_STOP)GetProcAddress(hDll,"i2c_stop")) == NULL){
			FreeLibrary(hDll);
			return FALSE;
		}
		if((i2c_write = (I2C_WRITE)GetProcAddress(hDll,"i2c_write")) == NULL){
			FreeLibrary(hDll);
			return FALSE;
		}
		if((i2c_read = (I2C_READ)GetProcAddress(hDll,"i2c_read")) == NULL){
			FreeLibrary(hDll);
			return FALSE;
		}
		if((i2c_connect = (I2C_CONNECT)GetProcAddress(hDll,"i2c_connect")) == NULL){
			FreeLibrary(hDll);
			return FALSE;
		}
		if((i2c_disconnect = (I2C_DISCONNECT)GetProcAddress(hDll,"i2c_disconnect")) == NULL){
			FreeLibrary(hDll);
			return FALSE;
		}
	}

	return TRUE;
}

////////////////////////////////////////////////////////////////////////////
BYTE reg_read(BYTE addr)
{
	BYTE rcv;

	rcv = 0;

	i2c_start();
	i2c_write(bDeviceAddr | IIC_WRITE);	//address
	i2c_write(addr);				//register
	i2c_start();
	i2c_write(0xce | IIC_READ);		//read
	rcv = i2c_read(0);
	i2c_stop();

	return rcv;
}

////////////////////////////////////////////////////////////////////////////
void reg_write(BYTE addr, BYTE dat)
{
	i2c_start();
	i2c_write(bDeviceAddr | IIC_WRITE);	//address
	i2c_write(addr);				//register
	//i2c_start();
	i2c_write(dat);		//read
	//rcv = i2c_read(0);
	i2c_stop();
}

////////////////////////////////////////////////////////////////////////////
void set_freq(DWORD freq)
{
	BYTE	mod;
	BYTE	cex;

	// CEXlI
	if (freq < 9000){
		cex = 3;
	} else if (freq < 9600){
		cex = 2;
	} else if (freq < 10200){
		cex = 1;
	} else {
		cex = 0;
	}
	// WX^ݒlvZ
	freq *= 10000;
	freq += 304000;
	freq /= (8192 / 2);
	mod = (BYTE)freq & 1;	// ÔP
	freq /= 2;
	freq += mod;

	reg_write(0x0E, 0x05);			//0b00000101 Zbg
	reg_write(0x01, 0xb4);			//0b10110100 Pilot ON, Forced subcarrier
	reg_write(0x02, 0x07);			//0b00000111 ULD ON, 2mW
#if 1
	reg_write(0x03, (BYTE)freq);		// g
	reg_write(0x04, (BYTE)(freq >> 8));	// g
	reg_write(0x08, 0x18 + cex);	// Local OSC Band
	//reg_write(0x08, 0x18 + 3);	// Local OSC Band
#else
	reg_write(0x03, 0b10001010);		// g
	reg_write(0x04, 0b00101110);	// g
	reg_write(0x08, 0b00011010);	// Local OSC Band
#endif
	reg_write(0x00, 0x01);			//0b00000001 Audio input 100mV, Pre-emphasis 50us, Power ON

	reg_write(0x0E, 0x05);			//0b00000101 Zbg
	reg_write(0x06, 0x1e);			//0b00011110 Charge pump
}

////////////////////////////////////////////////////////////////////////////
void print_freq(DWORD freq)
{
	putch((freq / 10000) + '0');
	freq %= 10000;

	putch((freq / 1000) + '0');
	freq %= 1000;

	putch((freq / 100) + '0');
	freq %= 100;

	putch('.');

	putch((freq / 10) + '0');
	freq %= 10;

	putch(freq + '0');

	printf("MHz\r");
}

////////////////////////////////////////////////////////////////////////////
void clear_keybuffer(void)
{
	for(;;){
		if(_kbhit() == 0){
			break;
		}
		else{
			_getch();
		}
	}
}

////////////////////////////////////////////////////////////////////////////
int main(int argc, char **argv)
{
	int iPort;
	int iKey;
	DWORD	freq = 8800;

	if(argc < 2){
		printf("usage : pressmain port-number\n");
		return -1;
	}

	if(LibInitialize() == FALSE){
		printf("DLL Error\n");
		return -1;
	}

	iPort = atoi(argv[1]);
	if(iPort == 0){
		printf("Port Number Error(%d)\n", iPort);
		return -1;
	}

	if(i2c_connect(iPort) == FALSE){
		printf("Port Open Error.\n");
		return -1;
	}

	printf("Esc:Exit 'U':Freq-Up 'D':Freq-Down\n");

	set_freq(freq);
	print_freq(freq);
	Sleep(50);

	for(;;){
		if(_kbhit() != 0){
			iKey = _getch();
			if(iKey == 0x1b){
				break;
			}
			else if((iKey == 'u') || (iKey == 'U')){
				if(freq < 10800){
					freq += 10;

					set_freq(freq);
					print_freq(freq);
					Sleep(10);
				}
			}
			else if((iKey == 'd') || (iKey == 'D')){
				if(freq > 8750){
					freq -= 10;

					set_freq(freq);
					print_freq(freq);
					Sleep(10);
				}
			}
			else if(iKey == 0xe0){
				iKey = _getch();
				switch(iKey){
				case 0x48:	//Up.
				case 0x4d:	//Right.
					if(freq < 10800){
						freq += 10;

						set_freq(freq);
						print_freq(freq);
						Sleep(10);
					}
					break;
				case 0x50:	//Down.
				case 0x4b:	//Left.
					if(freq > 8750){
						freq -= 10;

						set_freq(freq);
						print_freq(freq);
						Sleep(10);
					}
					break;
				}
			}
		}
		Sleep(SLEEP_TIME);
	}

	printf("\n");

	i2c_disconnect();

	clear_keybuffer();

	return 0;
}

