////////////////////////////////////////////////////////////////////////////
//
// SerialPort to I2C-LCD(AQM0802A) Test Program.
//
//   Copyright by rubidesu 2013
//
//   http://www.gatelink.co.jp/hw/
//
//   Version 1.00 2013.09.02
//           1.10 2013.09.18
//
////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <windows.h>
#include <conio.h>

#include "i2c.h"

BYTE bDeviceAddr = 0x7c;		//AQM0802A
#define RS_DATA		0x40
#define RS_CMND		0x00

const BYTE cgram_pat0[48] = {
	//CGRAM(1)     #####
	0x00, //0b00000,	// _____
	0x00, //0b00000,	// _____
	0x00, //0b00000,	// _____
	0x00, //0b00000,	// _____
	0x00, //0b00000,	// _____
	0x00, //0b00000,	// _____
	0x00, //0b00000,	// _____
	0x00, //0b00000,	// _____
	//CGRAM(2)     #####
	0x10, //0b10000,	// *____
	0x10, //0b10000,	// *____
	0x10, //0b10000,	// *____
	0x10, //0b10000,	// *____
	0x10, //0b10000,	// *____
	0x10, //0b10000,	// *____
	0x10, //0b10000,	// *____
	0x10, //0b10000,	// *____
	//CGRAM(3)     #####
	0x18, //0b11000,	// **___
	0x18, //0b11000,	// **___
	0x18, //0b11000,	// **___
	0x18, //0b11000,	// **___
	0x18, //0b11000,	// **___
	0x18, //0b11000,	// **___
	0x18, //0b11000,	// **___
	0x18, //0b11000,	// **___
	//CGRAM(4)     #####
	0x1c, //0b11100,	// ***__
	0x1c, //0b11100,	// ***__
	0x1c, //0b11100,	// ***__
	0x1c, //0b11100,	// ***__
	0x1c, //0b11100,	// ***__
	0x1c, //0b11100,	// ***__
	0x1c, //0b11100,	// ***__
	0x1c, //0b11100,	// ***__
	//CGRAM(5)     #####
	0x1e, //0b11110,	// ****_
	0x1e, //0b11110,	// ****_
	0x1e, //0b11110,	// ****_
	0x1e, //0b11110,	// ****_
	0x1e, //0b11110,	// ****_
	0x1e, //0b11110,	// ****_
	0x1e, //0b11110,	// ****_
	0x1e, //0b11110,	// ****_
	//CGRAM(6)     #####
	0x1f, //0b11111,	// *****
	0x1f, //0b11111,	// *****
	0x1f, //0b11111,	// *****
	0x1f, //0b11111,	// *****
	0x1f, //0b11111,	// *****
	0x1f, //0b11111,	// *****
	0x1f, //0b11111,	// *****
	0x1f, //0b11111,	// *****
};

////////////////////////////////////////////////////////////////////////////
BOOL LibInitialize()
{
	HINSTANCE hDll;

	if((hDll = LoadLibrary("i2c.dll")) == NULL){
		return FALSE;
	}
	else{
		if((i2c_start = (I2C_START)GetProcAddress(hDll,"i2c_start")) == NULL){
			FreeLibrary(hDll);
			return FALSE;
		}
		if((i2c_stop = (I2C_STOP)GetProcAddress(hDll,"i2c_stop")) == NULL){
			FreeLibrary(hDll);
			return FALSE;
		}
		if((i2c_write = (I2C_WRITE)GetProcAddress(hDll,"i2c_write")) == NULL){
			FreeLibrary(hDll);
			return FALSE;
		}
		if((i2c_read = (I2C_READ)GetProcAddress(hDll,"i2c_read")) == NULL){
			FreeLibrary(hDll);
			return FALSE;
		}
		if((i2c_connect = (I2C_CONNECT)GetProcAddress(hDll,"i2c_connect")) == NULL){
			FreeLibrary(hDll);
			return FALSE;
		}
		if((i2c_disconnect = (I2C_DISCONNECT)GetProcAddress(hDll,"i2c_disconnect")) == NULL){
			FreeLibrary(hDll);
			return FALSE;
		}
	}

	return TRUE;
}

////////////////////////////////////////////////////////////////////////////
void i2c_lcd_write(BYTE rs, BYTE dat)
{
	i2c_start();
	i2c_write(bDeviceAddr | IIC_WRITE);	//address
	i2c_write(0x80 | rs);
	i2c_write(dat);
	i2c_stop();
}

////////////////////////////////////////////////////////////////////////////
void i2c_lcd_clear(void)
{
	i2c_lcd_write(RS_CMND, 0x01);
	Sleep(10);
}

////////////////////////////////////////////////////////////////////////////
void i2c_lcd_putch(BYTE c)
{
	i2c_lcd_write(RS_DATA, c);
}

////////////////////////////////////////////////////////////////////////////
void i2c_lcd_goto(BYTE pos)
{
	i2c_lcd_write(RS_CMND, 0x80+pos);
}

////////////////////////////////////////////////////////////////////////////
void i2c_lcd_init(void)
{
	i2c_lcd_write(RS_CMND, 0x38 /*0b00111000*/);	//0x38 function set.
	i2c_lcd_write(RS_CMND, 0x39 /*0b00111001*/);	//0x39 function set.
	i2c_lcd_write(RS_CMND, 0x14 /*0b00010100*/);	//0x14 internal osc frequency.
	i2c_lcd_write(RS_CMND, 0x70 /*0b01110000*/);	//0x70 contrast set.
	i2c_lcd_write(RS_CMND, 0x56 /*0b01010110*/);	//0x56 power/icon/contrast control.
	i2c_lcd_write(RS_CMND, 0x6C /*0b01101100*/);	//0x6C follower control.
	Sleep(210);
	i2c_lcd_write(RS_CMND, 0x38 /*0b00111000*/);	//0x38 function set.
	i2c_lcd_write(RS_CMND, 0x0C /*0b00001100*/);	//0x0C display on/off control.
	i2c_lcd_write(RS_CMND, 0x01 /*0b00000001*/);	//0x01 clear display.
	Sleep(2);
}

////////////////////////////////////////////////////////////////////////////
void i2c_lcd_set_cgram(void)
{
	BYTE i;

	i2c_lcd_write(RS_CMND, 0x40);

	for(i = 0; i < 48; i++){
		i2c_lcd_write(RS_DATA, cgram_pat0[i]);
	}
}

////////////////////////////////////////////////////////////////////////////
void clear_keybuffer(void)
{
	for(;;){
		if(_kbhit() == 0){
			break;
		}
		else{
			_getch();
		}
	}
}

////////////////////////////////////////////////////////////////////////////
int main(int argc, char **argv)
{
	int iPort;
	int i;
	char cScreenStr[20];
	SYSTEMTIME stSystemTime;

	if(argc < 2){
		printf("usage : lcdmain port-number\n");
		return -1;
	}

	if(LibInitialize() == FALSE){
		printf("DLL Error\n");
		return -1;
	}

	iPort = atoi(argv[1]);
	if(iPort == 0){
		printf("Port Number Error(%d)\n", iPort);
		return -1;
	}

	if(i2c_connect(iPort) == FALSE){
		printf("Port Open Error.\n");
		return -1;
	}

	i2c_lcd_init();
	//i2c_lcd_set_cgram();
	i2c_lcd_clear();

	printf("It will end, if a key is pressed.\n");

	for(;;){
		if(_kbhit() != 0){
			break;
		}

		GetLocalTime(&stSystemTime);
		stSystemTime.wYear -= 2000;

		sprintf(cScreenStr, "%02d/%02d/%02d %02d:%02d:%02d",
			stSystemTime.wYear,
			stSystemTime.wMonth,
			stSystemTime.wDay,
			stSystemTime.wHour,
			stSystemTime.wMinute,
			stSystemTime.wSecond);

		i2c_lcd_goto(0x00);
		for(i = 0; i < 8; i++){
			i2c_lcd_putch(cScreenStr[i]);
		}

		i2c_lcd_goto(0x40);
		for(i = 0; i < 8; i++){
			i2c_lcd_putch(cScreenStr[i + 9]);
		}

		printf("%s\r", cScreenStr);

		Sleep(1000);
	}

	printf("\n");

	i2c_disconnect();

	clear_keybuffer();

	return 0;
}

