////////////////////////////////////////////////////////////////////////////
//
// SerialPort to I2C-Pressure Sensor(LPS331) and LCD(AQM0802A) Test Program.
//
//   Copyright by rubidesu 2013
//
//   http://www.gatelink.co.jp/hw/
//
//   Version 1.00 2013.09.02
//           1.10 2013.09.18
//
////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <windows.h>
#include <conio.h>

#include "i2c.h"

const BYTE bSensDeviceAddr = 0xb8;	//SAO(SDO) : Vss.
//const BYTE bSensDeviceAddr = 0xba;	//SAO(SDO) : Vdd.

#define RCV_IND_MAX	2
const BYTE bRcvInd[RCV_IND_MAX] = {'-', '|'};
static BYTE bRcvIndPnt = 0;

#undef PERFORMANCE_CHECK
//#define PERFORMANCE_CHECK

#ifdef PERFORMANCE_CHECK
#define PERFORM_TIMER_SET	TRUE
#define PERFORM_TIMER_GET	FALSE
static LARGE_INTEGER liFreq;
#endif //PERFORMANCE_CHECK

BYTE bLcdDeviceAddr = 0x7c;		//AQM0802A
#define RS_DATA		0x40
#define RS_CMND		0x00

////////////////////////////////////////////////////////////////////////////
BOOL LibInitialize()
{
	HINSTANCE hDll;

	if((hDll = LoadLibrary("i2c.dll")) == NULL){
		return FALSE;
	}
	else{
		if((i2c_start = (I2C_START)GetProcAddress(hDll,"i2c_start")) == NULL){
			FreeLibrary(hDll);
			return FALSE;
		}
		if((i2c_stop = (I2C_STOP)GetProcAddress(hDll,"i2c_stop")) == NULL){
			FreeLibrary(hDll);
			return FALSE;
		}
		if((i2c_write = (I2C_WRITE)GetProcAddress(hDll,"i2c_write")) == NULL){
			FreeLibrary(hDll);
			return FALSE;
		}
		if((i2c_read = (I2C_READ)GetProcAddress(hDll,"i2c_read")) == NULL){
			FreeLibrary(hDll);
			return FALSE;
		}
		if((i2c_connect = (I2C_CONNECT)GetProcAddress(hDll,"i2c_connect")) == NULL){
			FreeLibrary(hDll);
			return FALSE;
		}
		if((i2c_disconnect = (I2C_DISCONNECT)GetProcAddress(hDll,"i2c_disconnect")) == NULL){
			FreeLibrary(hDll);
			return FALSE;
		}
	}

	return TRUE;
}

////////////////////////////////////////////////////////////////////////////
BYTE reg_read(BYTE addr)
{
	BYTE rcv = 0;

	i2c_start();
	i2c_write(bSensDeviceAddr | IIC_WRITE);
	i2c_write(addr);
	i2c_start();
	i2c_write(bSensDeviceAddr | IIC_READ);
	rcv = i2c_read(IIC_NACK);
	i2c_stop();

	return rcv;
}

////////////////////////////////////////////////////////////////////////////
void reg_write(BYTE addr, BYTE dat)
{
	i2c_start();
	i2c_write(bSensDeviceAddr | IIC_WRITE);
	i2c_write(addr);
	i2c_write(dat);
	i2c_stop();
}

////////////////////////////////////////////////////////////////////////////
const BYTE cgram_pat0[64] = {
	//CGRAM(1)     #####
	0x08, //0b01000,	// _*___
	0x14, //0b10100,	// *_*__
	0x08, //0b01000,	// _*___
	0x07, //0b00111,	// __***
	0x08, //0b01000,	// _*___
	0x08, //0b01000,	// _*___
	0x07, //0b00111,	// __***
	0x00, //0b00000,	// _____
	//CGRAM(2)     #####
	0x08, //0b01000,	// _*___
	0x14, //0b10100,	// *_*__
	0x08, //0b01000,	// _*___
	0x07, //0b00111,	// __***
	0x08, //0b01000,	// _*___
	0x08, //0b01000,	// _*___
	0x07, //0b00111,	// __***
	0x00, //0b00000,	// _____
	//CGRAM(3)     #####
	0x18, //0b11000,	// **___
	0x18, //0b11000,	// **___
	0x18, //0b11000,	// **___
	0x18, //0b11000,	// **___
	0x18, //0b11000,	// **___
	0x18, //0b11000,	// **___
	0x18, //0b11000,	// **___
	0x18, //0b11000,	// **___
	//CGRAM(4)     #####
	0x1c, //0b11100,	// ***__
	0x1c, //0b11100,	// ***__
	0x1c, //0b11100,	// ***__
	0x1c, //0b11100,	// ***__
	0x1c, //0b11100,	// ***__
	0x1c, //0b11100,	// ***__
	0x1c, //0b11100,	// ***__
	0x1c, //0b11100,	// ***__
	//CGRAM(5)     #####
	0x1e, //0b11110,	// ****_
	0x1e, //0b11110,	// ****_
	0x1e, //0b11110,	// ****_
	0x1e, //0b11110,	// ****_
	0x1e, //0b11110,	// ****_
	0x1e, //0b11110,	// ****_
	0x1e, //0b11110,	// ****_
	0x1e, //0b11110,	// ****_
	//CGRAM(6)     #####
	0x1f, //0b11111,	// *****
	0x1f, //0b11111,	// *****
	0x1f, //0b11111,	// *****
	0x1f, //0b11111,	// *****
	0x1f, //0b11111,	// *****
	0x1f, //0b11111,	// *****
	0x1f, //0b11111,	// *****
	0x1f, //0b11111,	// *****
};

////////////////////////////////////////////////////////////////////////////
void i2c_lcd_write(BYTE rs, BYTE dat)
{
	i2c_start();
	i2c_write(bLcdDeviceAddr | IIC_WRITE);	//address
	i2c_write(0x80 | rs);
	i2c_write(dat);
	i2c_stop();
}

////////////////////////////////////////////////////////////////////////////
void i2c_lcd_clear(void)
{
	i2c_lcd_write(RS_CMND, 0x01);
	Sleep(10);
}

////////////////////////////////////////////////////////////////////////////
void i2c_lcd_putch(BYTE c)
{
	i2c_lcd_write(RS_DATA, c);
}

////////////////////////////////////////////////////////////////////////////
void i2c_lcd_goto(BYTE pos)
{
	i2c_lcd_write(RS_CMND, 0x80+pos);
}

////////////////////////////////////////////////////////////////////////////
void i2c_lcd_init(void)
{
	i2c_lcd_write(RS_CMND, 0x38 /*0b00111000*/);	//0x38 function set.
	i2c_lcd_write(RS_CMND, 0x39 /*0b00111001*/);	//0x39 function set.
	i2c_lcd_write(RS_CMND, 0x14 /*0b00010100*/);	//0x14 internal osc frequency.
	i2c_lcd_write(RS_CMND, 0x70 /*0b01110000*/);	//0x70 contrast set.
	i2c_lcd_write(RS_CMND, 0x56 /*0b01010110*/);	//0x56 power/icon/contrast control.
	i2c_lcd_write(RS_CMND, 0x6C /*0b01101100*/);	//0x6C follower control.
	Sleep(210);
	i2c_lcd_write(RS_CMND, 0x38 /*0b00111000*/);	//0x38 function set.
	i2c_lcd_write(RS_CMND, 0x0C /*0b00001100*/);	//0x0C display on/off control.
	i2c_lcd_write(RS_CMND, 0x01 /*0b00000001*/);	//0x01 clear display.
	Sleep(2);
}

////////////////////////////////////////////////////////////////////////////
void i2c_lcd_set_cgram(void)
{
	BYTE i;

	i2c_lcd_write(RS_CMND, 0x40);

	for(i = 0; i < 48; i++){
		i2c_lcd_write(RS_DATA, cgram_pat0[i]);
	}
}

////////////////////////////////////////////////////////////////////////////
void clear_keybuffer(void)
{
	for(;;){
		if(_kbhit() == 0){
			break;
		}
		else{
			_getch();
		}
	}
}

////////////////////////////////////////////////////////////////////////////
#ifdef PERFORMANCE_CHECK
DWORD perform_timer(BOOL boFlag)
{
	static LARGE_INTEGER liLastTimer;
	LARGE_INTEGER liTimer;

	if(boFlag == PERFORM_TIMER_SET){
		QueryPerformanceCounter(&liLastTimer);
		return 0L;
	}
	else{
		QueryPerformanceCounter(&liTimer);
		return (liTimer.QuadPart - liLastTimer.QuadPart) * 1000 / liFreq.QuadPart;
	}

	return 0L;
}
#endif //PERFORMANCE_CHECK

////////////////////////////////////////////////////////////////////////////
int main(int argc, char **argv)
{
	int iPort;
	BYTE read_data;
	DWORD dwPress = 0;
	WORD wTemp = 0;
	double dblPress;
	double dblTemp;
	char cPrintStr[16];
	int i;

	if(argc < 2){
		printf("usage : pressmain port-number\n");
		return -1;
	}

	if(LibInitialize() == FALSE){
		printf("DLL Error\n");
		return -1;
	}

	iPort = atoi(argv[1]);
	if(iPort == 0){
		printf("Port Number Error(%d)\n", iPort);
		return -1;
	}

	if(i2c_connect(iPort) == FALSE){
		printf("Port Open Error.\n");
		return -1;
	}

#ifdef PERFORMANCE_CHECK
	memset((void*)&liFreq, 0x00, sizeof(LARGE_INTEGER));
	QueryPerformanceFrequency(&liFreq);
#endif //PERFORMANCE_CHECK

	printf("It will end, if a key is pressed.\n");

	printf("WHO_AM_I Register:%02X\n", reg_read(0x0f));

	i2c_lcd_init();
	i2c_lcd_set_cgram();
	i2c_lcd_clear();

	reg_write(0x20, 0x90);

	for(;;){
		Sleep(1500);

		if(_kbhit() != 0){
			break;
		}

#ifdef PERFORMANCE_CHECK
		perform_timer(PERFORM_TIMER_SET);
#endif //PERFORMANCE_CHECK

		// C̎擾 ///////////////////////////////////////////
		read_data = reg_read(0x2a);
		//printf("%02X ", read_data);
		dwPress = ((DWORD)read_data << 16);

		read_data = reg_read(0x29);
		//printf("%02X ", read_data);
		dwPress = dwPress | ((DWORD)read_data << 8);

		read_data = reg_read(0x28);
		//printf("%02X ", read_data);
		dwPress = dwPress | ((DWORD)read_data & (DWORD)0xff);

		dblPress = (double)dwPress;
		dblPress = dblPress / 4096.0;

		// x̎擾 ///////////////////////////////////////////
		read_data = reg_read(0x2c);
		//printf("%02X ", read_data);
		wTemp = ((WORD)read_data << 8);

		read_data = reg_read(0x2b);
		//printf("%02X ", read_data);
		wTemp = wTemp | ((WORD)read_data & (WORD)0xff);

		dblTemp = (double)((short)wTemp / 480.0 + 42.5);

		//printf("\r\n");

		// LCD̕` ///////////////////////////////////////////
		sprintf(cPrintStr, "P%7.2f", dblPress);
		i2c_lcd_goto(0x00);
		for(i = 0; i < strlen(cPrintStr); i++){
			i2c_lcd_putch(cPrintStr[i]);
		}
		sprintf(cPrintStr, "T%6.2f%c", dblTemp, 1);
		cPrintStr[1] = cPrintStr[1] == ' ' ? '+' : cPrintStr[1];
		i2c_lcd_goto(0x40);
		for(i = 0; i < strlen(cPrintStr); i++){
			i2c_lcd_putch(cPrintStr[i]);
		}

#ifdef PERFORMANCE_CHECK
		printf("%7.2fhPa %6.2fC %4ldmsec. %c    \r",
			dblPress, dblTemp, perform_timer(PERFORM_TIMER_GET), bRcvInd[bRcvIndPnt]);
#else //PERFORMANCE_CHECK
		printf("%7.2fhPa %6.2fC %c    \r",
			dblPress, dblTemp, bRcvInd[bRcvIndPnt]);
#endif //PERFORMANCE_CHECK

		bRcvIndPnt++;
		bRcvIndPnt %= RCV_IND_MAX;
	}

	printf("\n");

	i2c_disconnect();

	clear_keybuffer();

	return 0;
}

