////////////////////////////////////////////////////////////////////////////
//
// SerialPort to I2C-Pressure Sensor(LPS331) Test Program.
//
//   Copyright by rubidesu 2013
//
//   http://www.gatelink.co.jp/hw/
//
//   Version 1.00 2013.09.02
//           1.10 2013.09.18
//
////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <windows.h>
#include <conio.h>

#include "i2c.h"

const BYTE bSensDeviceAddr = 0xb8;	//SAO(SDO) : Vss.
//const BYTE bSensDeviceAddr = 0xba;	//SAO(SDO) : Vdd.

#define RCV_IND_MAX	2
const BYTE bRcvInd[RCV_IND_MAX] = {'-', '|'};
static BYTE bRcvIndPnt = 0;

#undef PERFORMANCE_CHECK
//#define PERFORMANCE_CHECK

#ifdef PERFORMANCE_CHECK
#define PERFORM_TIMER_SET	TRUE
#define PERFORM_TIMER_GET	FALSE
static LARGE_INTEGER liFreq;
#endif //PERFORMANCE_CHECK

////////////////////////////////////////////////////////////////////////////
BOOL LibInitialize()
{
	HINSTANCE hDll;

	if((hDll = LoadLibrary("i2c.dll")) == NULL){
		return FALSE;
	}
	else{
		if((i2c_start = (I2C_START)GetProcAddress(hDll,"i2c_start")) == NULL){
			FreeLibrary(hDll);
			return FALSE;
		}
		if((i2c_stop = (I2C_STOP)GetProcAddress(hDll,"i2c_stop")) == NULL){
			FreeLibrary(hDll);
			return FALSE;
		}
		if((i2c_write = (I2C_WRITE)GetProcAddress(hDll,"i2c_write")) == NULL){
			FreeLibrary(hDll);
			return FALSE;
		}
		if((i2c_read = (I2C_READ)GetProcAddress(hDll,"i2c_read")) == NULL){
			FreeLibrary(hDll);
			return FALSE;
		}
		if((i2c_connect = (I2C_CONNECT)GetProcAddress(hDll,"i2c_connect")) == NULL){
			FreeLibrary(hDll);
			return FALSE;
		}
		if((i2c_disconnect = (I2C_DISCONNECT)GetProcAddress(hDll,"i2c_disconnect")) == NULL){
			FreeLibrary(hDll);
			return FALSE;
		}
	}

	return TRUE;
}

////////////////////////////////////////////////////////////////////////////
BYTE reg_read(BYTE addr)
{
	BYTE rcv = 0;

	i2c_start();
	i2c_write(bSensDeviceAddr | IIC_WRITE);
	i2c_write(addr);
	i2c_start();
	i2c_write(bSensDeviceAddr | IIC_READ);
	rcv = i2c_read(IIC_NACK);
	i2c_stop();

	return rcv;
}

////////////////////////////////////////////////////////////////////////////
void reg_write(BYTE addr, BYTE dat)
{
	i2c_start();
	i2c_write(bSensDeviceAddr | IIC_WRITE);
	i2c_write(addr);
	i2c_write(dat);
	i2c_stop();
}

////////////////////////////////////////////////////////////////////////////
void clear_keybuffer(void)
{
	for(;;){
		if(_kbhit() == 0){
			break;
		}
		else{
			_getch();
		}
	}
}

////////////////////////////////////////////////////////////////////////////
#ifdef PERFORMANCE_CHECK
DWORD perform_timer(BOOL boFlag)
{
	static LARGE_INTEGER liLastTimer;
	LARGE_INTEGER liTimer;

	if(boFlag == PERFORM_TIMER_SET){
		QueryPerformanceCounter(&liLastTimer);
		return 0L;
	}
	else{
		QueryPerformanceCounter(&liTimer);
		return (liTimer.QuadPart - liLastTimer.QuadPart) * 1000 / liFreq.QuadPart;
	}

	return 0L;
}
#endif //PERFORMANCE_CHECK

////////////////////////////////////////////////////////////////////////////
int main(int argc, char **argv)
{
	int iPort;
	BYTE read_data;
	DWORD dwPress = 0;
	WORD wTemp = 0;
	double dblPress;
	double dblTemp;
	int i;

	if(argc < 2){
		printf("usage : pressmain port-number\n");
		return -1;
	}

	if(LibInitialize() == FALSE){
		printf("DLL Error\n");
		return -1;
	}

	iPort = atoi(argv[1]);
	if(iPort == 0){
		printf("Port Number Error(%d)\n", iPort);
		return -1;
	}

	if(i2c_connect(iPort) == FALSE){
		printf("Port Open Error.\n");
		return -1;
	}

#ifdef PERFORMANCE_CHECK
	memset((void*)&liFreq, 0x00, sizeof(LARGE_INTEGER));
	QueryPerformanceFrequency(&liFreq);
#endif //PERFORMANCE_CHECK

	printf("It will end, if a key is pressed.\n");

	printf("WHO_AM_I Register:");
	for(i = 0;; i++){
		read_data = reg_read(0x0f);
		printf("%02X ", read_data);
		if(read_data == 0xbb){
			break;
		}
		if(i >= 10){
			printf("\nSensor Error(%02X)\n", read_data);
			return -1;
		}
		Sleep(100);
	}
	printf("\n");

	reg_write(0x20, 0x90);

	for(;;){
		Sleep(1500);

		if(_kbhit() != 0){
			break;
		}

#ifdef PERFORMANCE_CHECK
		perform_timer(PERFORM_TIMER_SET);
#endif //PERFORMANCE_CHECK

		// C̎擾 ///////////////////////////////////////////
		read_data = reg_read(0x2a);
		dwPress = ((DWORD)read_data << 16);

		read_data = reg_read(0x29);
		dwPress = dwPress | ((DWORD)read_data << 8);

		read_data = reg_read(0x28);
		dwPress = dwPress | ((DWORD)read_data & (DWORD)0xff);

		dblPress = (double)dwPress;
		dblPress = dblPress / 4096.0;

		// x̎擾 ///////////////////////////////////////////
		read_data = reg_read(0x2c);
		wTemp = ((WORD)read_data << 8);

		read_data = reg_read(0x2b);
		wTemp = wTemp | ((WORD)read_data & (WORD)0xff);

		dblTemp = (double)((short)wTemp / 480.0 + 42.5);

#ifdef PERFORMANCE_CHECK
		printf("%7.2fhPa %6.2fC %4ldmsec. %c    \r",
			dblPress, dblTemp, perform_timer(PERFORM_TIMER_GET), bRcvInd[bRcvIndPnt]);
#else //PERFORMANCE_CHECK
		printf("%7.2fhPa %6.2fC %c    \r",
			dblPress, dblTemp, bRcvInd[bRcvIndPnt]);
#endif //PERFORMANCE_CHECK

		bRcvIndPnt++;
		bRcvIndPnt %= RCV_IND_MAX;
	}

	printf("\n");

	i2c_disconnect();

	clear_keybuffer();

	return 0;
}

