////////////////////////////////////////////////////////////////////////////
//
// SerialPort to I2C-Radio Module(TEA5767) Test Program.
//
//   Copyright by rubidesu 2013
//
//   http://www.gatelink.co.jp/hw/
//
//   Version 1.00 2013.09.02
//           1.10 2013.09.18
//
////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <windows.h>
#include <conio.h>

#include "i2c.h"

#define DEFAULT_FREQ	800

#define SLEEP_TIME		10		//msecP.
#define REFRESH_TIME	500		//msecP.
#define REFRESH_COUNT	(REFRESH_TIME/SLEEP_TIME)

WORD wRadioFreq = DEFAULT_FREQ;
BOOL bRefreshFlg = TRUE;

const BYTE bDeviceAddr = 0xc0;

////////////////////////////////////////////////////////////////////////////
BOOL LibInitialize()
{
	HINSTANCE hDll;

	if((hDll = LoadLibrary("i2c.dll")) == NULL){
		return FALSE;
	}
	else{
		if((i2c_start = (I2C_START)GetProcAddress(hDll,"i2c_start")) == NULL){
			FreeLibrary(hDll);
			return FALSE;
		}
		if((i2c_stop = (I2C_STOP)GetProcAddress(hDll,"i2c_stop")) == NULL){
			FreeLibrary(hDll);
			return FALSE;
		}
		if((i2c_write = (I2C_WRITE)GetProcAddress(hDll,"i2c_write")) == NULL){
			FreeLibrary(hDll);
			return FALSE;
		}
		if((i2c_read = (I2C_READ)GetProcAddress(hDll,"i2c_read")) == NULL){
			FreeLibrary(hDll);
			return FALSE;
		}
		if((i2c_connect = (I2C_CONNECT)GetProcAddress(hDll,"i2c_connect")) == NULL){
			FreeLibrary(hDll);
			return FALSE;
		}
		if((i2c_disconnect = (I2C_DISCONNECT)GetProcAddress(hDll,"i2c_disconnect")) == NULL){
			FreeLibrary(hDll);
			return FALSE;
		}
	}

	return TRUE;
}

////////////////////////////////////////////////////////////////////////////
void set_freq(void)
{
	DWORD val;

	val = ((((DWORD)wRadioFreq * 12500) + 28125)) / 1024;

	i2c_start();
	i2c_write(bDeviceAddr | IIC_WRITE);	//device address
	i2c_write(val >> 8);	//1byte
	i2c_write(val & 0xff);	//2byte
	i2c_write(0x10);	//3byte
	i2c_write(0x36);	//4byte
	i2c_write(0x00);	//5byte
	i2c_stop();
}

////////////////////////////////////////////////////////////////////////////
VOID get_info(BOOL *boStereo, BYTE *bAdcLevel)
{
	BYTE bReadData[5];

	i2c_start();
	i2c_write(bDeviceAddr | IIC_READ);	//device address
	bReadData[0] = i2c_read(IIC_ACK);	//1byte
	//printf("%02X ", bReadData);
	bReadData[1] = i2c_read(IIC_ACK);	//2byte
	//printf("%02X ", bReadData);
	bReadData[2] = i2c_read(IIC_ACK);	//3byte
	//printf("%02X ", bReadData);
	bReadData[3] = i2c_read(IIC_ACK);	//4byte
	//printf("%02X ", bReadData);
	bReadData[4] = i2c_read(IIC_NACK);	//5byte
	//printf("%02X\n", bReadData);
	i2c_stop();

	if((bReadData[2] & 0x80) == 0x80){
		*boStereo = TRUE;
	}
	else{
		*boStereo = FALSE;
	}

	*bAdcLevel = ((bReadData[3] & 0xf0) >> 4);
}

////////////////////////////////////////////////////////////////////////////
void print_level(BYTE bLevel)
{
	int i, j;
	for(i = 0; i < bLevel; i++){
		putch('|');
	}
	for(j = i; j < 16; j++){
		putch(' ');
	}
	putch('\r');
}

////////////////////////////////////////////////////////////////////////////
void clear_keybuffer(void)
{
	for(;;){
		if(_kbhit() == 0){
			break;
		}
		else{
			_getch();
		}
	}
}

////////////////////////////////////////////////////////////////////////////
void refresh_info(void)
{
	BOOL boStereo;
	BYTE bAdcLevel;

	get_info(&boStereo, &bAdcLevel);

	printf("%5.1fMHz %s %s %2d ",
		(double)wRadioFreq / 10.0,
		bRefreshFlg == TRUE ? "RE" : "  ",
		boStereo == TRUE ? "ST" : "MO",
		bAdcLevel);
	print_level(bAdcLevel);
}

////////////////////////////////////////////////////////////////////////////
int main(int argc, char **argv)
{
	int iPort;
	int iKey;
//	BOOL boStereo;
//	BYTE bAdcLevel;
	BYTE bRefreshCount = 0;

	if(argc < 2){
		printf("usage : pressmain port-number\n");
		return -1;
	}

	if(LibInitialize() == FALSE){
		printf("DLL Error\n");
		return -1;
	}

	iPort = atoi(argv[1]);
	if(iPort == 0){
		printf("Port Number Error(%d)\n", iPort);
		return -1;
	}

	if(i2c_connect(iPort) == FALSE){
		printf("Port Open Error.\n");
		return -1;
	}

	printf("Esc:Exit 'U':Freq-Up 'D':Freq-Down 'R':Refresh-ON/OFF.\n");

	set_freq();
	Sleep(50);
	refresh_info();

	for(;;){
		if(_kbhit() != 0){
			iKey = _getch();
			if(iKey == 0x1b){
				break;
			}
			else if((iKey == 'u') || (iKey == 'U')){
				if(wRadioFreq < 1080){
					wRadioFreq++;

					set_freq();
					Sleep(10);
					refresh_info();

					bRefreshCount = 0;
				}
			}
			else if((iKey == 'd') || (iKey == 'D')){
				if(wRadioFreq > 760){
					wRadioFreq--;

					set_freq();
					Sleep(10);
					refresh_info();

					bRefreshCount = 0;
				}
			}
			else if((iKey == 'r') || (iKey == 'R')){
				bRefreshFlg = !bRefreshFlg;
				refresh_info();
			}
			else if(iKey == 0xe0){
				iKey = _getch();
				switch(iKey){
				case 0x48:	//Up.
				case 0x4d:	//Right.
					if(wRadioFreq < 1080){
						wRadioFreq++;

						set_freq();
						Sleep(10);
						refresh_info();

						bRefreshCount = 0;
					}
					break;
				case 0x50:	//Down.
				case 0x4b:	//Left.
					if(wRadioFreq > 760){
						wRadioFreq--;

						set_freq();
						Sleep(10);
						refresh_info();

						bRefreshCount = 0;
					}
					break;
				}
			}
		}
		if(bRefreshCount > REFRESH_COUNT){
			if(bRefreshFlg == TRUE){
				refresh_info();
			}

			bRefreshCount = 0;
		}

		bRefreshCount++;

		Sleep(SLEEP_TIME);
	}

	printf("\n");

	i2c_disconnect();

	clear_keybuffer();

	return 0;
}

