////////////////////////////////////////////////////////////////////////////
//
// SerialPort to I2C-RTC(RTC-8564NB) Test Program.
//
//   Copyright by rubidesu 2013
//
//   http://www.gatelink.co.jp/hw/
//
//   Version 1.00 2013.09.02
//           1.10 2013.09.18
//
////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <windows.h>
#include <conio.h>

#include "i2c.h"

const BYTE bDeviceAddr = 0xa2;

const char cWeekDay[7][4] = {
	{"Sun"},
	{"Mon"},
	{"Tue"},
	{"Wed"},
	{"Shu"},
	{"Fri"},
	{"Sat"},
};

const unsigned char NUM_TABLE[] = {'0','1','2','3','4','5','6','7','8','9'};

////////////////////////////////////////////////////////////////////////////
BOOL LibInitialize()
{
	HINSTANCE hDll;

	if((hDll = LoadLibrary("i2c.dll")) == NULL){
		return FALSE;
	}
	else{
		if((i2c_start = (I2C_START)GetProcAddress(hDll,"i2c_start")) == NULL){
			FreeLibrary(hDll);
			return FALSE;
		}
		if((i2c_stop = (I2C_STOP)GetProcAddress(hDll,"i2c_stop")) == NULL){
			FreeLibrary(hDll);
			return FALSE;
		}
		if((i2c_write = (I2C_WRITE)GetProcAddress(hDll,"i2c_write")) == NULL){
			FreeLibrary(hDll);
			return FALSE;
		}
		if((i2c_read = (I2C_READ)GetProcAddress(hDll,"i2c_read")) == NULL){
			FreeLibrary(hDll);
			return FALSE;
		}
		if((i2c_connect = (I2C_CONNECT)GetProcAddress(hDll,"i2c_connect")) == NULL){
			FreeLibrary(hDll);
			return FALSE;
		}
		if((i2c_disconnect = (I2C_DISCONNECT)GetProcAddress(hDll,"i2c_disconnect")) == NULL){
			FreeLibrary(hDll);
			return FALSE;
		}
	}

	return TRUE;
}

////////////////////////////////////////////////////////////////////////////
void set_rtctime(BYTE bYear, BYTE bMonth, BYTE bDayOfWeek, BYTE bDay, BYTE bHour, BYTE bMinute, BYTE bSecond)
{
	i2c_start();
	i2c_write(bDeviceAddr | IIC_WRITE);
	i2c_write(0x02);
	i2c_write(bSecond); //sec
	i2c_write(bMinute); //min
	i2c_write(bHour); //hour
	i2c_write(bDay); //day
	i2c_write(bDayOfWeek); //weekday
	i2c_write(bMonth); //month
	i2c_write(bYear); //year
	i2c_stop();
}

////////////////////////////////////////////////////////////////////////////
void get_rtctime(BYTE *bYear, BYTE *bMonth, BYTE *bDayOfWeek, BYTE *bDay, BYTE *bHour, BYTE *bMinute, BYTE *bSecond)
{
	i2c_start();
	i2c_write(bDeviceAddr | IIC_WRITE);
	i2c_write(0x02);
	i2c_start();
	i2c_write(bDeviceAddr | IIC_READ);

	*bSecond = i2c_read(IIC_ACK) & 0x7f;
	*bMinute = i2c_read(IIC_ACK) & 0x7f;
	*bHour = i2c_read(IIC_ACK) & 0x3f;
	*bDay = i2c_read(IIC_ACK) & 0x3f;
	*bDayOfWeek = i2c_read(IIC_ACK) & 0x07;
	*bMonth = i2c_read(IIC_ACK) & 0x1f;
	*bYear = i2c_read(IIC_NACK) & 0xff;

	i2c_stop();
}

////////////////////////////////////////////////////////////////////////////
void put_hex_chr(unsigned char c)
{
	putch(NUM_TABLE[c >> 4]);
	putch(NUM_TABLE[c & 0x0f]);
}

////////////////////////////////////////////////////////////////////////////
void clear_keybuffer(void)
{
	for(;;){
		if(_kbhit() == 0){
			break;
		}
		else{
			_getch();
		}
	}
}

////////////////////////////////////////////////////////////////////////////
int main(int argc, char **argv)
{
	int iPort;
	SYSTEMTIME stSystemTime;
	BYTE bYear;
	BYTE bMonth;
	BYTE bDayOfWeek;
	BYTE bDay;
	BYTE bHour;
	BYTE bMinute;
	BYTE bSecond;

	if(argc < 2){
		printf("usage : pressmain port-number\n");
		return -1;
	}

	if(LibInitialize() == FALSE){
		printf("DLL Error\n");
		return -1;
	}

	iPort = atoi(argv[1]);
	if(iPort == 0){
		printf("Port Number Error(%d)\n", iPort);
		return -1;
	}

	if(i2c_connect(iPort) == FALSE){
		printf("Port Open Error.\n");
		return -1;
	}

	printf("It will end, if a key is pressed.\n");

	GetLocalTime(&stSystemTime);
	stSystemTime.wYear -= 2000;
	bYear = ((stSystemTime.wYear / 10) << 4) | (stSystemTime.wYear % 10);
	bMonth = ((stSystemTime.wMonth / 10) << 4) | (stSystemTime.wMonth % 10);
	bDay = ((stSystemTime.wDay / 10) << 4) | (stSystemTime.wDay % 10);
	bDayOfWeek = stSystemTime.wDayOfWeek;
	bHour = ((stSystemTime.wHour / 10) << 4) | (stSystemTime.wHour % 10);
	bMinute = ((stSystemTime.wMinute / 10) << 4) | (stSystemTime.wMinute % 10);
	bSecond = ((stSystemTime.wSecond / 10) << 4) | (stSystemTime.wSecond % 10);

	printf("Start Time:%02X/%02X/%02X(%s) %02X:%02X:%02X\n",
		bYear, bMonth, bDay, cWeekDay[bDayOfWeek], bHour, bMinute, bSecond);

	set_rtctime(bYear, bMonth, bDayOfWeek, bDay, bHour, bMinute, bSecond);

	for(;;){
		if(_kbhit() != 0){
			break;
		}

		get_rtctime(&bYear, &bMonth, &bDayOfWeek, &bDay, &bHour, &bMinute, &bSecond);

		printf("Now Time  :%02X/%02X/%02X(%s) %02X:%02X:%02X\r",
			bYear, bMonth, bDay, cWeekDay[bDayOfWeek], bHour, bMinute, bSecond);

		Sleep(1000);
	}

	printf("\n");

	i2c_disconnect();

	clear_keybuffer();

	return 0;
}

